(setq sound-alist nil)
(set-frame-size (selected-frame) 85 30) 
(set-frame-position (selected-frame) 0 0) 


        (setq uko-emacs-directory "\\UKO-II\\")
        (setq uko-physical-switch-0 [(kp-0)])
	(setq uko-physical-switch-1 [(kp-1)])
	(setq uko-physical-switch-2 [(kp-2)])
	(setq uko-physical-switch-3 [(kp-3)])
     	(load (concat uko-emacs-directory "lisp\\uko-ii.el"))
      (setq Info-directory-list
	      (append Info-directory-list (concat uko-emacs-directory "doc\\Benutzerhandbuch\\info\\"))
	)
